proc daily_d_ask_hour_notes {org_id proj_id daily_id year month day hour}\
{
	upvar #0 DAILY_REC_${org_id}_${proj_id}_${daily_id} DAILY_REC
	global notes
	catch {destroy .ask}
	toplevel .ask
	wm title .ask "Hour notes"
	wm geometry .ask +100+100
	label .ask.date -text "[gen_get_weekday_name $year $month $day] [gen_get_month_name $month] $day $year, $hour"
	pack .ask.date -side top -fill x -anchor w
	text .ask.notes -width 30 -height 2 -wrap word
	pack .ask.notes -side top -fill x -anchor w
	frame .ask.b
	pack .ask.b -side bottom
		button .ask.b.ok -text Ok -width 10 \
			-command {
				set notes [.ask.notes get 1.0 end-1c]
				destroy .ask
			}
		pack .ask.b.ok -side left
		button .ask.b.cancel -text Cancel -width 10
		pack .ask.b.cancel -side left
	if {[keylget DAILY_REC $year.$month.$day.$hour.notes {}]}\
	{
		.ask.notes insert 1.0 "[keylget DAILY_REC $year.$month.$day.$hour.notes]"
		.ask.b.cancel configure -command "
			set notes [list [keylget DAILY_REC $year.$month.$day.$hour.notes]]
			destroy .ask
		"
	}\
	else\
	{
		.ask.b.cancel configure -command "
			set notes \"\"
			destroy .ask
		"
	}
	focus .ask.notes
}

proc daily_popup_hour_notes {org_id proj_id daily_id year month day hour}\
{
	upvar #0 DAILY_REC_${org_id}_${proj_id}_${daily_id} DAILY_REC
	global notes
	daily_d_ask_hour_notes $org_id $proj_id $daily_id $year $month $day $hour
	tkwait variable notes
	if {$notes != ""}\
	{
		keylset DAILY_REC $year.$month.$day.$hour.notes "$notes"
		return "$notes"
	}\
	else\
	{
		if {[keylget DAILY_REC $year.$month.$day.$hour.notes {}]}\
		{
			keyldel DAILY_REC $year.$month.$day.$hour
		}
		return ""
	}
}

proc daily_create_day {org_id proj_id daily_id year month day}\
{
	upvar #0 DAILY_REC_${org_id}_${proj_id}_${daily_id} DAILY_REC
	upvar #0 DAILY_${org_id}_${proj_id}_${daily_id} DAILY
	global DAILY_COL
	if {[keylget DAILY_REC $year.$month.$day.type {}]}\
	{
		.r.daily.c create rectangle 0 0 300 30 -fill $DAILY_COL([keylget DAILY_REC $year.$month.$day.type]) -outline {}
		.r.daily.c create rectangle 0 0 50 760 -fill $DAILY_COL([keylget DAILY_REC $year.$month.$day.type]) -outline {}
	}\
	else\
	{
		.r.daily.c create rectangle 0 0 300 30 -fill #EEEEEE -outline {}
		.r.daily.c create rectangle 0 0 50 760 -fill #EEEEEE -outline {}
	}
	.r.daily.c create text 60 10 -text "[gen_get_weekday_name $year $month $day] [gen_get_month_name $month] $day $year" -anchor nw
	set j 50
	if {[keylget DAILY mode] == 12}\
	{
		set hours "7:30 8:00 8:30 9:00 9:30 10:00 10:30 11:00 11:30 12:00 12:30 13:00 13:30 14:00 14:30 15:00 15:30 16:00 16:30 17:00 17:30 18:00 18:30 19:00"
	}\
	else\
	{
		set hours "0:00 1:00 2:00 3:00 4:00 5:00 6:00 7:00 8:00 9:00 10:00 11:00 12:00 13:00 14:00 15:00 16:00 17:00 18:00 19:00 20:00 21:00 22:00 23:00"
	}
	foreach hour $hours \
	{
		set tag1 [.r.daily.c create text 40 $j -text "$hour" -anchor ne]
		if {[keylget DAILY_REC $year.$month.$day.$hour.notes {}]}\
		{
			set tag2 [.r.daily.c create text 60 $j -text "[keylget DAILY_REC $year.$month.$day.$hour.notes]" -width 300 -anchor nw]
		}\
		else\
		{
			set tag2 [.r.daily.c create text 60 $j -text "" -width 300 -anchor nw]
		}
		set cmd "daily_popup_hour_notes $org_id $proj_id $daily_id $year $month $day $hour"
		.r.daily.c bind $tag1 <1> "
			.r.daily.c itemconfigure $tag2 -text \"\[$cmd\]\"
		"
		.r.daily.c bind $tag2 <1> "
			.r.daily.c itemconfigure $tag2 -text \"\[$cmd\]\"
		"
		incr j 30
	}
	.r.daily.c configure -scrollregion [.r.daily.c bbox all]
}

proc daily_draw_month {ystart org_id proj_id daily_id year month day {selmode day} {selvalue 1}}\
{
	upvar #0 DAILY_REC_${org_id}_${proj_id}_${daily_id} DAILY_REC
	global DAILY_COL
	if {$selmode == "day"}\
	{
		.r.daily.c create rectangle 1 [expr 1 + $ystart] 150 [expr 170 + $ystart] -fill #FFFFFF
	}
	if {$selmode == "month" && $selvalue == $month}\
	{
		.r.daily.c create text 75 [expr 15 + $ystart] -text "[gen_get_month_name $month] $year" -fill #990000
	}\
	else\
	{
		.r.daily.c create text 75 [expr 15 + $ystart] -text "[gen_get_month_name $month] $year" -fill #000099
	}
	set pos [expr [gen_get_weekday $year $month 1] % 7]
	if {[lsearch "1 3 5 7 8 10 12" $month] >= 0}\
	{
		set end 31
	}\
	elseif {[lsearch "4 6 9 11" $month] >= 0}\
	{
		set end 30
	}\
	elseif {[expr $year % 4] == 0}\
	{
		set end 29
	}\
	else\
	{
		set end 28
	}
	set i 1
	.r.daily.c create rectangle 5 [expr 27 + $ystart] 147 [expr 45 + $ystart] -fill #FFFFEE -outline {}
	foreach wd "S M T W T F S" \
	{
		.r.daily.c create text [expr 20*$i + 4] [expr 30 + $ystart] -text $wd -anchor ne -fill #990000
		incr i
	}
	loop d 1 [expr $end + 1] \
	{
		if {[keylget DAILY_REC $year.$month.$d.type {}] && [keylget DAILY_REC $year.$month.$d.type] > 1}\
		{
			.r.daily.c create rectangle [expr 20*($pos % 7) + 26] [expr 50 + 20*int($pos/7) + $ystart - 3] [expr 20*($pos % 7) + 6] [expr 50 + 20*int($pos/7) + $ystart + 17] -fill $DAILY_COL([keylget DAILY_REC $year.$month.$d.type]) -outline {} -tags r$d
		}
		if {$selmode == "day" && $d == $day}\
		{
			.r.daily.c create text [expr 20*($pos % 7) + 24] [expr 50 + 20*int($pos/7) + $ystart] -text $d -anchor ne -fill red -tags "day d$d"
		}\
		else\
		{
			.r.daily.c create text [expr 20*($pos % 7) + 24] [expr 50 + 20*int($pos/7) + $ystart] -text $d -anchor ne -tags "day d$d"
		}
		incr pos
	}
	.r.daily.c configure -scrollregion [.r.daily.c bbox all]
}

proc daily_d_ask_day_notes {org_id proj_id daily_id year month day}\
{
	upvar #0 DAILY_REC_${org_id}_${proj_id}_${daily_id} DAILY_REC
	global DAILY_COL notes type
	if {[keylget DAILY_REC $year.$month.$day.type {}]}\
	{
		set type [keylget DAILY_REC $year.$month.$day.type]
	}\
	else\
	{
		set type 1
	}
	catch {destroy .ask}
	toplevel .ask
	wm title .ask "Day notes"
	wm geometry .ask +100+100
	label .ask.date -text "[gen_get_month_name $month] $day $year"
	pack .ask.date -side top -fill x -anchor w
	text .ask.notes -width 30 -height 8 -wrap word
	pack .ask.notes -side top -fill x -anchor w
	frame .ask.type
	pack .ask.type -side top
		radiobutton .ask.type.t1 -text None -variable type -value 1
		pack .ask.type.t1 -side left
		radiobutton .ask.type.t2 -background $DAILY_COL(2) -variable type -value 2
		pack .ask.type.t2 -side left
		radiobutton .ask.type.t3 -background $DAILY_COL(3) -variable type -value 3
		pack .ask.type.t3 -side left
		radiobutton .ask.type.t4 -background $DAILY_COL(4) -variable type -value 4
		pack .ask.type.t4 -side left
		radiobutton .ask.type.t5 -background $DAILY_COL(5) -variable type -value 5
		pack .ask.type.t5 -side left
	frame .ask.b
	pack .ask.b -side bottom
		button .ask.b.ok -text Ok -width 10 \
			-command {
				set notes [.ask.notes get 1.0 end-1c]
				destroy .ask
			}
		pack .ask.b.ok -side left
		button .ask.b.cancel -text Cancel -width 10
		pack .ask.b.cancel -side left
	if {[keylget DAILY_REC $year.$month.$day.notes {}]}\
	{
		.ask.notes insert 1.0 "[keylget DAILY_REC $year.$month.$day.notes]"
		.ask.b.cancel configure -command "
			set type $type
			set notes [list [keylget DAILY_REC $year.$month.$day.notes]]
			destroy .ask
		"
	}\
	else\
	{
		.ask.b.cancel configure -command "
			set type $type
			set notes \"\"
			destroy .ask
		"
	}
	focus .ask.notes
}

proc daily_popup_day_notes {org_id proj_id daily_id year month day}\
{
	upvar #0 DAILY_REC_${org_id}_${proj_id}_${daily_id} DAILY_REC
	global DAILY_COL notes type
	daily_d_ask_day_notes $org_id $proj_id $daily_id $year $month $day
	tkwait variable notes
	if {$type > 1}\
	{
		keylset DAILY_REC $year.$month.$day.type $type
		set pos [expr [gen_get_weekday $year $month 1] % 7]
		set pos [expr $pos + $day - 1]
		.r.daily.c delete r$day
		set tag [.r.daily.c create rectangle [expr 20*($pos % 7) + 26] [expr 50 + 20*int($pos/7) - 3] [expr 20*($pos % 7) + 6] [expr 50 + 20*int($pos/7) + 17] -fill $DAILY_COL([keylget DAILY_REC $year.$month.$day.type]) -outline {} -tags r$day]
		.r.daily.c raise d$day
	}\
	else\
	{
		if {[keylget DAILY_REC $year.$month.$day.type {}]}\
		{
			keyldel DAILY_REC $year.$month.$day.type
		}
		.r.daily.c delete r$day
	}
	if {$notes != ""}\
	{
		keylset DAILY_REC $year.$month.$day.notes "$notes"
		return "$notes"
	}\
	else\
	{
		if {[keylget DAILY_REC $year.$month.$day.notes {}]}\
		{
			keyldel DAILY_REC $year.$month.$day.notes
		}
		return ""
	}
}

proc daily_get_day_notes {org_id proj_id daily_id year month day}\
{
	upvar #0 DAILY_REC_${org_id}_${proj_id}_${daily_id} DAILY_REC
	if {[keylget DAILY_REC $year.$month.$day.notes {}]}\
	{
		return [keylget DAILY_REC $year.$month.$day.notes]
	}\
	else\
	{
		return ""
	}
}

proc daily_create_month {org_id proj_id daily_id year month day}\
{
	daily_draw_month 0 $org_id $proj_id $daily_id $year $month $day day $day
	set tag1 [.r.daily.c create text 160 10 -text "" -fill #990000 -anchor nw]
	set tag2 [.r.daily.c create text 160 30 -text "" -width 250p -anchor nw]
	for {set d 1} {$d <= 31} {incr d}\
	{
		set t [lindex [.r.daily.c gettags d$d] 1]
		.r.daily.c bind $t <Any-Enter> "
			.r.daily.c itemconfigure $tag1 -text \"[gen_get_month_name $month] $d\"
			.r.daily.c itemconfigure $tag2 -text \"\[daily_get_day_notes $org_id $proj_id $daily_id $year $month $d\]\"
		"
		.r.daily.c bind $t <1> "
			.r.daily.c itemconfigure $tag1 -text \"[gen_get_month_name $month] $d\"
			.r.daily.c itemconfigure $tag2 -text \"\[daily_popup_day_notes $org_id $proj_id $daily_id $year $month $d\]\"
		"
	}
}

proc daily_d_ask_month_notes {org_id proj_id daily_id year month}\
{
	upvar #0 DAILY_REC_${org_id}_${proj_id}_${daily_id} DAILY_REC
	global notes
	catch {destroy .ask}
	toplevel .ask
	wm title .ask "Month notes"
	wm geometry .ask +100+100
	label .ask.date -text "[gen_get_month_name $month] $year"
	pack .ask.date -side top -fill x -anchor w
	text .ask.notes -width 30 -height 8 -wrap word
	pack .ask.notes -side top -fill x -anchor w
	frame .ask.b
	pack .ask.b -side bottom
		button .ask.b.ok -text Ok -width 10 \
			-command {
				set notes [.ask.notes get 1.0 end-1c]
				destroy .ask
			}
		pack .ask.b.ok -side left
		button .ask.b.cancel -text Cancel -width 10
		pack .ask.b.cancel -side left
	if {[keylget DAILY_REC $year.$month.notes {}]}\
	{
		.ask.notes insert 1.0 "[keylget DAILY_REC $year.$month.notes]"
		.ask.b.cancel configure -command "
			set notes [list [keylget DAILY_REC $year.$month.notes]]
			destroy .ask
		"
	}\
	else\
	{
		.ask.b.cancel configure -command "
			set notes \"\"
			destroy .ask
		"
	}
	focus .ask.notes
}

proc daily_popup_month_notes {org_id proj_id daily_id year month}\
{
	upvar #0 DAILY_REC_${org_id}_${proj_id}_${daily_id} DAILY_REC
	global notes
	daily_d_ask_month_notes $org_id $proj_id $daily_id $year $month
	tkwait variable notes
	if {$notes != ""}\
	{
		keylset DAILY_REC $year.$month.notes $notes
		return "$notes"
	}\
	else\
	{
		if {[keylget DAILY_REC $year.$month.notes {}]}\
		{
			keyldel DAILY_REC $year.$month.notes
		}
		return ""
	}
}

proc daily_create_year {org_id proj_id daily_id year month day}\
{
	upvar #0 DAILY_REC_${org_id}_${proj_id}_${daily_id} DAILY_REC
	for {set j 0} {$j < 12} {incr j}\
	{
		daily_draw_month [expr $j * 170] $org_id $proj_id $daily_id $year [expr $j + 1] $day month $month
		set tagrect [.r.daily.c create rectangle 160 [expr $j * 170 + 25] 385 [expr ($j + 1) *170 - 5] -fill #EEFFFF -outline {}]
		if {[keylget DAILY_REC $year.[expr $j + 1].notes {}]}\
		{
			set tag [.r.daily.c create text 165 [expr $j * 170 + 30] -text "[keylget DAILY_REC $year.[expr $j + 1].notes]" -anchor nw -width 200p]
		}\
		else\
		{
			set tag [.r.daily.c create text 165 [expr $j * 170 + 30] -text "" -anchor nw -width 200p]
		}
		set cmd "daily_popup_month_notes $org_id $proj_id $daily_id $year [expr $j + 1]"
		.r.daily.c bind $tag <1> "
			.r.daily.c itemconfigure $tag -text \"\[$cmd\]\"
		"
		.r.daily.c bind $tagrect <1> "
			.r.daily.c itemconfigure $tag -text \"\[$cmd\]\"
		"
	}
}

proc daily_create_calendar {org_id proj_id daily_id mode year month day}\
{
	o_wait
	.r.daily.c delete all
	if {[cindex $month 0] == "0"}\
	{
		set month [cindex $month 1]
	}
	if {[cindex $day 0] == "0"}\
	{
		set day [cindex $day 1]
	}
	switch $mode \
	{
		day 	{
				daily_create_day $org_id $proj_id $daily_id $year $month $day
		}
		month	{
				daily_create_month $org_id $proj_id $daily_id $year $month $day
		}
		year	{
				daily_create_year $org_id $proj_id $daily_id $year $month $day
		}
	}
	o_idle
}

###########################

proc daily_init_var {org_id proj_id daily_id}\
{
	global DAILY_REC_${org_id}_${proj_id}_${daily_id}
	global CUR_ID_${org_id}_${proj_id}_${daily_id}
	global CUR_NUM_${org_id}_${proj_id}_${daily_id}
	global TOT_NUM_${org_id}_${proj_id}_${daily_id}
	set DAILY_REC_${org_id}_${proj_id}_${daily_id} ""
	set CUR_ID_${org_id}_${proj_id}_${daily_id} 0
	set CUR_NUM_${org_id}_${proj_id}_${daily_id} 1
	set TOT_NUM_${org_id}_${proj_id}_${daily_id} 1
}

proc daily_new_id {org_id proj_id}\
{
	global LIST_DAILY
	set daily_id 0
	foreach b $LIST_DAILY \
	{
		if {[regexp "^${org_id}_${proj_id}_(\[0-9\]+)" $b all id]}\
		{
			if {$id > $daily_id}\
			{
				set daily_id $id
			}
		}
	}
	return [expr $daily_id + 1]
}

proc daily_d_new {org_id}\
{
	global CUR_ORG LIST_PROJ LIST_DAILY
	global proj mode
	if {! [info exists mode]}\
	{
		set mode 12
	}
	catch {destroy .newdaily}
	toplevel .newdaily
	wm title .newdaily "New planner"
	wm geometry .newdaily +50+50
	frame .newdaily.b
	pack .newdaily.b -side bottom -pady 5 -padx 5
		button .newdaily.b.ok -text Ok -width 10 \
			-command {
				if {[.newdaily.l.f3.lb size] == 0}\
				{
					return
				}
				set proj_id [join [lindex [split $proj ---] 0]]
				set daily_id [daily_new_id $CUR_ORG $proj_id]
				daily_init_var $CUR_ORG $proj_id $daily_id
				upvar #0 DAILY_REC_${CUR_ORG}_${proj_id}_${daily_id} DAILY_REC
				set DAILY_REC ""
				upvar #0 DAILY_${CUR_ORG}_${proj_id}_${daily_id} DAILY
				set DAILY ""
				if {[join [.newdaily.name.e get]] == ""}\
				{
					keylset DAILY name "Untitled"
				}\
				else\
				{
					keylset DAILY name [.newdaily.name.e get]
				}
				keylset DAILY mode $mode
				lappend LIST_DAILY ${CUR_ORG}_${proj_id}_${daily_id}
				daily_save $CUR_ORG $proj_id $daily_id
				o_menu_compose $CUR_ORG
				destroy .newdaily
			}
		pack .newdaily.b.ok -side left
		button .newdaily.b.cancel -text Cancel -width 10 \
			-command "
				destroy .newdaily
			"
		pack .newdaily.b.cancel -side left
	frame .newdaily.name
	pack .newdaily.name -fill x
		label .newdaily.name.l -text "Planner's name: "
		pack .newdaily.name.l -side left
		entry .newdaily.name.e
		pack .newdaily.name.e -side left
	frame .newdaily.mode
	pack .newdaily.mode -side top
		radiobutton .newdaily.mode.m12 -text "12 hours / 30 min" -variable mode -value 12
		pack .newdaily.mode.m12 -side left
		radiobutton .newdaily.mode.m24 -text "24 hours / 60 min" -variable mode -value 24
		pack .newdaily.mode.m24 -side left
	frame .newdaily.l
	pack .newdaily.l -side left -fill y -pady 5 -padx 5
		frame .newdaily.l.f3
		pack .newdaily.l.f3
			scrollbar .newdaily.l.f3.vs -command {.newdaily.l.f3.lb yview}
			pack .newdaily.l.f3.vs -side right -fill y
			listbox .newdaily.l.f3.lb -width 30 -yscrollcommand {.newdaily.l.f3.vs set}
			pack .newdaily.l.f3.lb -side right -fill y
			foreach p $LIST_PROJ \
			{
				global PROJ_${org_id}_$p
				.newdaily.l.f3.lb insert end "$p --- [keylget PROJ_${org_id}_$p name]"
			}
			.newdaily.l.f3.lb selection set 0
			set proj [.newdaily.l.f3.lb get 0]
			bind .newdaily.l.f3.lb <1> {
				update
				set proj [.newdaily.l.f3.lb get @%x,%y]
			}
	focus .newdaily.name.e
}

proc daily_d_delete_before {org_id proj_id daily_id}\
{
	global ORG_PATH LIST_DAILY CUR_DAILY CUR_TYPE
	global CUR_YEAR CUR_MONTH CUR_DAY DAILY_MODE
	upvar #0 DAILY_REC_${org_id}_${proj_id}_${daily_id} DAILY_REC
	catch {destroy .ask}
	toplevel .ask
	wm title .ask "Delete items before..."
	wm geometry .ask +100+100
	frame .ask.date
	pack .ask.date -side top
		label .ask.date.lb -text "Date (YYYY MM DD)" -width 20
		pack .ask.date.lb -side left
		entry .ask.date.year -width 4
		pack .ask.date.year -side left
		.ask.date.year insert 0 $CUR_YEAR
		entry .ask.date.month -width 2
		pack .ask.date.month -side left
		.ask.date.month insert 0 $CUR_MONTH
		entry .ask.date.day -width 2
		pack .ask.date.day -side left
		.ask.date.day insert 0 $CUR_DAY
	frame .ask.b
	pack .ask.b -side bottom
		button .ask.b.ok -text Ok -width 10 \
			-command "
				if {\[gen_check_date \[.ask.date.year get\] \[.ask.date.month get\] \[.ask.date.day get\] \"Date\"\]}\
				{
					regsub -all \"^0*\" \[.ask.date.year get\] \"\" y
					regsub -all \"^0*\" \[.ask.date.month get\] \"\" m
					regsub -all \"^0*\" \[.ask.date.day get\] \"\" d
					#set date \[format \"%04d-%02d-%02d\" \$y \$m \$d\]
					upvar #0 DAILY_REC_${org_id}_${proj_id}_${daily_id} DAILY_REC
					foreach year \[keylkeys DAILY_REC\]\
					{
						if {\$year < \$y}\
						{
							keyldel DAILY_REC \$year
						}
					}
					if {\[keylget DAILY_REC \$y {}\]}\
					{
						foreach month \[keylkeys DAILY_REC \$y\]\
						{
							if {\$month < \$m}\
							{
								keyldel DAILY_REC \$y.\$month
							}
						}
					}
					if {\[keylget DAILY_REC \$y.\$m {}\]}\
					{
						foreach day \[keylkeys DAILY_REC \$y.\$m\]\
						{
							if {\$day < \$d}\
							{
								keyldel DAILY_REC \$y.\$m.\$day
							}
						}
					}
					o_display_daily $org_id $proj_id $daily_id $DAILY_MODE \[gen_get_cur_year\] \[gen_get_cur_month\] \[gen_get_cur_day\]
					destroy .ask
				}
			"
		pack .ask.b.ok -side left
		button .ask.b.cancel -text Cancel -width 10 -command {destroy .ask}
		pack .ask.b.cancel -side left
}

proc daily_delete {org_id proj_id daily_id}\
{
	global ORG_PATH LIST_DAILY CUR_DAILY CUR_TYPE
	global DAILY_${org_id}_${proj_id}_${daily_id}
	global DAILY_REC_${org_id}_${proj_id}_${daily_id}
	global confirm
	gen_d_yn_confirm "Do you really want to delete \"[keylget DAILY_${org_id}_${proj_id}_${daily_id} name]\"?"
	tkwait variable confirm
	if {$confirm == "yes"}\
	{
		unlink $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/daily_${org_id}_${proj_id}_${daily_id}
		set LIST_DAILY [lindex [intersect3 $LIST_DAILY ${org_id}_${proj_id}_${daily_id}] 0]
		unset DAILY_${org_id}_${proj_id}_${daily_id}
		unset DAILY_REC_${org_id}_${proj_id}_${daily_id}
		o_menu_compose $org_id
		set CUR_TYPE proj
		o_display_proj $org_id $proj_id
	}
}

###########################

proc daily_save {org_id proj_id daily_id}\
{
	upvar #0 DAILY_${org_id}_${proj_id}_${daily_id} DAILY
	upvar #0 DAILY_REC_${org_id}_${proj_id}_${daily_id} DAILY_REC
	global ORG_PATH
	o_wait
	set ct [catch {
		set F [open $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/daily_${org_id}_${proj_id}_${daily_id}.tmp w+]
	} errmsg]
	if {$ct > 0}\
	{
		myError $errmsg
		return
	}
	set ct [catch {
	puts $F "set DAILY_${org_id}_${proj_id}_${daily_id} \"\""
	puts $F "\n# STRUCTURE"
	puts $F "keylset DAILY_${org_id}_${proj_id}_${daily_id} name [list [keylget DAILY name]]"
	puts $F "keylset DAILY_${org_id}_${proj_id}_${daily_id} mode \"[keylget DAILY mode]\""
	puts $F "\n# RECORDS"
	puts $F "set DAILY_REC_${org_id}_${proj_id}_${daily_id} \"\"\n"
	foreach year [keylkeys DAILY_REC] \
	{
		foreach month [keylkeys DAILY_REC $year] \
		{
			foreach day [keylkeys DAILY_REC $year.$month] \
			{
				if {$day != "notes"}\
				{
					foreach hour [keylkeys DAILY_REC $year.$month.$day] \
					{
						if {$hour != "notes" && $hour != "type"}\
						{
							puts $F "keylset DAILY_REC_${org_id}_${proj_id}_${daily_id} $year.$month.$day.$hour.notes [list [keylget DAILY_REC $year.$month.$day.$hour.notes]]"
						}\
						else\
						{
							puts $F "keylset DAILY_REC_${org_id}_${proj_id}_${daily_id} $year.$month.$day.$hour [list [keylget DAILY_REC $year.$month.$day.$hour]]"
						}
					}
				}\
				else\
				{
					puts $F "keylset DAILY_REC_${org_id}_${proj_id}_${daily_id} $year.$month.$day [list [keylget DAILY_REC $year.$month.$day]]"
				}
			}
		}
	}
	close $F
	} errmsg]
	if {$ct > 0}\
	{
		unlink $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/daily_${org_id}_${proj_id}_${daily_id}.tmp
		myError "Cannot save Planner. Reason:\n$errmsg"
	}\
	else\
	{
		frename $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/daily_${org_id}_${proj_id}_${daily_id}.tmp $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/daily_${org_id}_${proj_id}_${daily_id}
		chmod 0600 $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/daily_${org_id}_${proj_id}_${daily_id}
	}
	o_idle
}

proc daily_export_ascii {org_id proj_id daily_id}\
{
	upvar #0 DAILY_${org_id}_${proj_id}_${daily_id} DAILY
	upvar #0 DAILY_REC_${org_id}_${proj_id}_${daily_id} DAILY_REC
	global FILENAME DAILY_MODE CUR_YEAR CUR_MONTH CUR_DAY
	o_wait
	daily_save $org_id $proj_id $daily_id
	regsub "^0*" $CUR_MONTH "" CUR_MONTH
	regsub "^0*" $CUR_DAY "" CUR_DAY
	set ct [catch {
		set F [open [pwd]/$FILENAME w+]
	} errmsg]
	if {$ct > 0}\
	{
		myError "Cannot save the planner as text. Reason:\n$errmsg"
		return
	}
	puts $F "[keylget DAILY name]\n"
	switch $DAILY_MODE \
	{
		day		{
			puts $F "[gen_get_weekday_name $CUR_YEAR $CUR_MONTH $CUR_DAY] [gen_get_month_name $CUR_MONTH] $CUR_DAY $CUR_YEAR\n"
			if {[keylget DAILY mode] == 12}\
			{
				set hours "7:30 8:00 8:30 9:00 9:30 10:00 10:30 11:00 11:30 12:00 12:30 13:00 13:30 14:00 14:30 15:00 15:30 16:00 16:30 17:00 17:30 18:00 18:30 19:00"
			}\
			else\
			{
				set hours "0:00 1:00 2:00 3:00 4:00 5:00 6:00 7:00 8:00 9:00 10:00 11:00 12:00 13:00 14:00 15:00 16:00 17:00 18:00 19:00 20:00 21:00 22:00 23:00"
			}
			foreach hour $hours \
			{
				set fhour [format "    %5s  " $hour]
				if {[keylget DAILY_REC $CUR_YEAR.$CUR_MONTH.$CUR_DAY.$hour.notes {}]}\
				{
					set notes [keylget DAILY_REC $CUR_YEAR.$CUR_MONTH.$CUR_DAY.$hour.notes]
					regsub -all "\n" $notes "\n            " notes
					puts $F "$fhour $notes"
				}\
				else\
				{
					puts $F "$fhour"
				}
			}
		}
		month	{
			puts $F "[gen_get_month_name $CUR_MONTH] $CUR_YEAR\n"
			set pos [gen_get_weekday $CUR_YEAR $CUR_MONTH 1]
			set line ""
			for {set i 0} {$i < $pos} {incr i}\
			{
				append line "   "
			}
			set d 1
			while {$pos <= 6}\
			{
				append line [format "%2s " $d]
				incr d
				incr pos
			}
			puts $F "$line"
			set end [gen_get_day $CUR_YEAR $CUR_MONTH 31]
			while {$d <= $end}\
			{
				set pos 0
				set line ""
				while {$pos <= 6 && $d <= $end}\
				{
					append line [format "%2d " $d]
					incr d
					incr pos
				}
				puts $F "$line"
			}
			puts $F ""
			for {set day 1} {$day <= [gen_get_day $CUR_YEAR $CUR_MONTH 31]} {incr day}\
			{
				set fday [format "    %2s  " $day]
				if {[keylget DAILY_REC $CUR_YEAR.$CUR_MONTH.$day.notes {}]}\
				{
					
					set notes [keylget DAILY_REC $CUR_YEAR.$CUR_MONTH.$day.notes]
					regsub -all "\n" $notes "\n         " notes
					puts $F "$fday $notes"
				}\
				else\
				{
					puts $F "$fday"
				}
			}
		}
		year	{
			foreach month "1 2 3 4 5 6 7 8 9 10 11 12" \
			{
				puts $F "        [gen_get_month_name $month] $CUR_YEAR"
				set pos [gen_get_weekday $CUR_YEAR $month 1]
				set line "  "
				for {set i 0} {$i < $pos} {incr i}\
				{
					append line "   "
				}
				set d 1
				while {$pos <= 6}\
				{
					append line [format "%2s " $d]
					incr d
					incr pos
				}
				puts $F "$line"
				set end [gen_get_day $CUR_YEAR $month 31]
				while {$d <= $end}\
				{
					set pos 0
					set line "  "
					while {$pos <= 6 && $d <= $end}\
					{
						append line [format "%2d " $d]
						incr d
						incr pos
					}
					puts $F "$line"
				}
				puts $F ""
				if {[keylget DAILY_REC $CUR_YEAR.$month.notes {}]}\
				{
					puts $F "[keylget DAILY_REC $CUR_YEAR.$month.notes]"
				}
				puts $F "-------------------------"
			}
		}
	}
	close $F
	o_idle
}

proc daily_export_html {org_id proj_id daily_id}\
{
	upvar #0 DAILY_${org_id}_${proj_id}_${daily_id} DAILY
	upvar #0 DAILY_REC_${org_id}_${proj_id}_${daily_id} DAILY_REC
	global FILENAME DAILY_MODE DAILY_COL CUR_YEAR CUR_MONTH CUR_DAY
	o_wait
	daily_save $org_id $proj_id $daily_id
	regsub "^0*" $CUR_MONTH "" CUR_MONTH
	regsub "^0*" $CUR_DAY "" CUR_DAY
	if {[file extension $FILENAME] != ".html"}\
	{
		append FILENAME ".html"
	}
	set ct [catch {
		set F [open [pwd]/$FILENAME w+]
	} errmsg]
	if {$ct > 0}\
	{
		myError "Cannot save the planner as text. Reason:\n$errmsg"
		return
	}
	puts $F "<html><head>"
	puts $F "<title>[html_text2html [keylget DAILY name]]</title></head>"
	puts $F "<body bgcolor=\"#FFFFFF\">"
	puts $F "<h1>[html_text2html [keylget DAILY name]]</h1>"
	switch $DAILY_MODE \
	{
		day		{
			puts $F "<h2>[gen_get_weekday_name $CUR_YEAR $CUR_MONTH $CUR_DAY] [gen_get_month_name $CUR_MONTH] $CUR_DAY $CUR_YEAR</h2>"
			if {[keylget DAILY mode] == 12}\
			{
				set hours "7:30 8:00 8:30 9:00 9:30 10:00 10:30 11:00 11:30 12:00 12:30 13:00 13:30 14:00 14:30 15:00 15:30 16:00 16:30 17:00 17:30 18:00 18:30 19:00"
			}\
			else\
			{
				set hours "0:00 1:00 2:00 3:00 4:00 5:00 6:00 7:00 8:00 9:00 10:00 11:00 12:00 13:00 14:00 15:00 16:00 17:00 18:00 19:00 20:00 21:00 22:00 23:00"
			}
			if {[keylget DAILY_REC $CUR_YEAR.$CUR_MONTH.$CUR_DAY.type {}]}\
			{
				set bgcolor $DAILY_COL([keylget DAILY_REC $CUR_YEAR.$CUR_MONTH.$CUR_DAY.type])
			}\
			else\
			{
				set bgcolor $DAILY_COL(1)
			}
			puts $F "<table>"
			foreach hour $hours \
			{
				if {[keylget DAILY_REC $CUR_YEAR.$CUR_MONTH.$CUR_DAY.$hour.notes {}]}\
				{
					regsub -all "\n" [keylget DAILY_REC $CUR_YEAR.$CUR_MONTH.$CUR_DAY.$hour.notes] "<br>" notes
					puts $F "\t<tr><td align=\"right\" bgcolor=\"$bgcolor\"><b>$hour</b></td><td>[html_text2html $notes]</td></tr>"
				}\
				else\
				{
					puts $F "\t<tr><td align=\"right\" bgcolor=\"$bgcolor\"><b>$hour</b></td><td>&nbsp;</td></tr>"
				}
			}
			puts $F "</table>"
			puts $F "</body></html>"
		}
		month	{
			puts $F "<h2>[gen_get_month_name $CUR_MONTH] $CUR_YEAR</h2>"
			set pos [gen_get_weekday $CUR_YEAR $CUR_MONTH 1]
			puts $F "<table>"
			set line "\t<tr>"
			for {set i 0} {$i < $pos} {incr i}\
			{
				append line "<td align=\"right\">&nbsp;</td>"
			}
			set d 1
			while {$pos <= 6}\
			{
				if {[keylget DAILY_REC $CUR_YEAR.$CUR_MONTH.$d.type {}]}\
				{
					append line "<td align=\"right\" bgcolor=\"$DAILY_COL([keylget DAILY_REC $CUR_YEAR.$CUR_MONTH.$d.type])\">$d</td>"
				}\
				else\
				{
					append line "<td align=\"right\">$d</td>"
				}
				incr d
				incr pos
			}
			puts $F "$line</tr>"
			set end [gen_get_day $CUR_YEAR $CUR_MONTH 31]
			while {$d <= $end}\
			{
				set pos 0
				set line "\t<tr>"
				while {$pos <= 6 && $d <= $end}\
				{
					if {[keylget DAILY_REC $CUR_YEAR.$CUR_MONTH.$d.type {}]}\
					{
						append line "<td align=\"right\" bgcolor=\"$DAILY_COL([keylget DAILY_REC $CUR_YEAR.$CUR_MONTH.$d.type])\">$d</td>"
					}\
					else\
					{
						append line "<td align=\"right\">$d</td>"
					}
					incr d
					incr pos
				}
				puts $F "$line</tr>"
			}
			puts $F "</table>"
			puts $F "<table>"
			for {set day 1} {$day <= [gen_get_day $CUR_YEAR $CUR_MONTH 31]} {incr day}\
			{
				if {[keylget DAILY_REC $CUR_YEAR.$CUR_MONTH.$day.notes {}]}\
				{
					if {[keylget DAILY_REC $CUR_YEAR.$CUR_MONTH.$day.type {}]}\
					{
						regsub -all "\n" [keylget DAILY_REC $CUR_YEAR.$CUR_MONTH.$day.notes] "<br>" notes
						puts $F "\t<tr><td align=\"right\" bgcolor=\"$DAILY_COL([keylget DAILY_REC $CUR_YEAR.$CUR_MONTH.$day.type])\"><b>$day</b></td><td>[html_text2html $notes]</td></tr>"
					}\
					else\
					{
						regsub -all "\n" [keylget DAILY_REC $CUR_YEAR.$CUR_MONTH.$day.notes] "<br>" notes
						puts $F "\t<tr><td align=\"right\"><b>$day</b></td><td>[html_text2html $notes]</td></tr>"
					}
				}\
				else\
				{
					if {[keylget DAILY_REC $CUR_YEAR.$CUR_MONTH.$day.type {}]}\
					{
						puts $F "\t<tr><td align=\"right\" bgcolor=\"$DAILY_COL([keylget DAILY_REC $CUR_YEAR.$CUR_MONTH.$day.type])\"><b>$day</b></td><td>&nbsp;</td></tr>"
					}\
					else\
					{
						puts $F "\t<tr><td align=\"right\"><b>$day</b></td><td>&nbsp;</td></tr>"
					}
				}
			}
			puts $F "</table>"
			puts $F "</body></html>"
		}
		year	{
			foreach month "1 2 3 4 5 6 7 8 9 10 11 12" \
			{
				puts $F "<h2>[gen_get_month_name $month] $CUR_YEAR</h2>"
				set pos [gen_get_weekday $CUR_YEAR $month 1]
				puts $F "<table>"
				set line "\t<tr>"
				for {set i 0} {$i < $pos} {incr i}\
				{
					append line "<td align=\"right\">&nbsp;</td>"
				}
				set d 1
				while {$pos <= 6}\
				{
					if {[keylget DAILY_REC $CUR_YEAR.$month.$d.type {}]}\
					{
						append line "<td align=\"right\" bgcolor=\"$DAILY_COL([keylget DAILY_REC $CUR_YEAR.$month.$d.type])\">$d</td>"
					}\
					else\
					{
						append line "<td align=\"right\">$d</td>"
					}
					incr d
					incr pos
				}
				puts $F "$line</tr>"
				set end [gen_get_day $CUR_YEAR $month 31]
				while {$d <= $end}\
				{
					set pos 0
					set line "\<tr>"
					while {$pos <= 6 && $d <= $end}\
					{
						if {[keylget DAILY_REC $CUR_YEAR.$month.$d.type {}]}\
						{
							append line "<td align=\"right\" bgcolor=\"$DAILY_COL([keylget DAILY_REC $CUR_YEAR.$month.$d.type])\">$d</td>"
						}\
						else\
						{
							append line "<td align=\"right\">$d</td>"
						}
						incr d
						incr pos
					}
					puts $F "$line</tr>"
				}
				puts $F "</table>"
				if {[keylget DAILY_REC $CUR_YEAR.$month.notes {}]}\
				{
					regsub -all "\n" [keylget DAILY_REC $CUR_YEAR.$month.notes] "<br>" notes
					puts $F "[html_text2html $notes]"
				}
			}
			puts $F "</body></html>"
		}
	}
	close $F
	o_idle
}

proc daily_d_export {org_id proj_id daily_id mode}\
{
	global FILENAME fbe 
	catch {destroy .file}
	toplevel .file
	wm title .file "Export planner - $mode"
	wm geometry .file 250x250+100+50
	filebrowser .file.fb 
	pack .file.fb
		if {$mode == "html"}\
		{
			.file.fb addlistbox .*.html -selectmode single
		}\
		else\
		{
			.file.fb addlistbox .* -selectmode single
		}
		.file.fb addlabel -text "Export: "
		.file.fb adddirlabel
		set fbe [.file.fb addentry -width 20]
		.file.fb addbutton -text OK -width 10 \
			-command "
				set FILENAME \[$fbe get\]
				if {\$FILENAME != \"\"}\
				{
					destroy .file
					daily_export_$mode $org_id $proj_id $daily_id
				}
			"
		.file.fb addbutton -text Cancel -width 10 \
			-command "destroy .file"
		if {$mode == "html"}\
		{
			.file.fb readdir [pwd] .*.html
		}\
		else\
		{
			.file.fb readdir [pwd] .*
		}
	focus $fbe
}

