proc notes_init_var {org_id proj_id notes_id}\
{
	global NOTES_${org_id}_${proj_id}_${notes_id}
	set NOTES_${org_id}_${proj_id}_${notes_id} ""
}

proc notes_new_id {org_id proj_id}\
{
	global LIST_NOTES
	set notes_id 0
	foreach b $LIST_NOTES \
	{
		if {[regexp "^${org_id}_${proj_id}_(\[0-9\]+)" $b all id]}\
		{
			if {$id > $notes_id}\
			{
				set notes_id $id
			}
		}
	}
	return [expr $notes_id + 1]
}

proc notes_d_new {org_id}\
{
	global proj
	global CUR_ORG LIST_PROJ LIST_NOTES
	catch {destroy .newnotes}
	toplevel .newnotes
	wm title .newnotes "New notes"
	wm geometry .newnotes +50+50
	frame .newnotes.b
	pack .newnotes.b -side bottom -pady 5 -padx 5
		button .newnotes.b.ok -text Ok -width 10 \
			-command {
				if {[.newnotes.l.f3.lb size] == 0}\
				{
					return
				}
				set proj_id [join [lindex [split $proj ---] 0]]
				set notes_id [notes_new_id $CUR_ORG $proj_id]
				notes_init_var $CUR_ORG $proj_id $notes_id
				upvar #0 NOTES_${CUR_ORG}_${proj_id}_${notes_id} NOTES
				set NOTES ""
				if {[join [.newnotes.name.e get]] == ""}\
				{
					keylset NOTES name "Untitled"
				}\
				else\
				{
					keylset NOTES name [.newnotes.name.e get]
				}
				keylset NOTES notes ""
				lappend LIST_NOTES ${CUR_ORG}_${proj_id}_${notes_id}
				notes_save $CUR_ORG $proj_id $notes_id
				o_menu_compose $CUR_ORG
				destroy .newnotes
			}
		pack .newnotes.b.ok -side left
		button .newnotes.b.cancel -text Cancel -width 10 \
			-command "
				destroy .newnotes
			"
		pack .newnotes.b.cancel -side left
	frame .newnotes.name
	pack .newnotes.name -fill x
		label .newnotes.name.l -text "Notes name: "
		pack .newnotes.name.l -side left
		entry .newnotes.name.e
		pack .newnotes.name.e -side left
	frame .newnotes.l
	pack .newnotes.l -side left -fill y -pady 5 -padx 5
		frame .newnotes.l.f3
		pack .newnotes.l.f3
			scrollbar .newnotes.l.f3.vs -command {.newnotes.l.f3.lb yview}
			pack .newnotes.l.f3.vs -side right -fill y
			listbox .newnotes.l.f3.lb -width 30 -yscrollcommand {.newnotes.l.f3.vs set}
			pack .newnotes.l.f3.lb -side right -fill y
			foreach p $LIST_PROJ \
			{
				global PROJ_${org_id}_$p
				.newnotes.l.f3.lb insert end "$p --- [keylget PROJ_${org_id}_$p name]"
			}
			.newnotes.l.f3.lb selection set 0
			set proj [.newnotes.l.f3.lb get 0]
			bind .newnotes.l.f3.lb <1> {
				update
				set proj [.newnotes.l.f3.lb get @%x,%y]
			}
	focus .newnotes.name.e
}

proc notes_save {org_id proj_id notes_id}\
{
	upvar #0 NOTES_${org_id}_${proj_id}_${notes_id} NOTES
	global ORG_PATH
	set ct [catch {
		set F [open $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/notes_${org_id}_${proj_id}_${notes_id}.tmp w+]
	} errmsg]
	if {$ct > 0}\
	{
		myError "$errmsg"
		return
	}
	set ct [catch {
		puts $F "# STRUCTURE\n"
		puts $F "set NOTES_${org_id}_${proj_id}_${notes_id} \"\"\n"
		puts $F "keylset NOTES_${org_id}_${proj_id}_${notes_id} name [list [keylget NOTES name]]\n"
		puts $F "keylset NOTES_${org_id}_${proj_id}_${notes_id} notes [list [keylget NOTES notes]]\n"
		close $F
	} errmsg]
	if {$ct > 0}\
	{
		unlink $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/notes_${org_id}_${proj_id}_${notes_id}.tmp
		myError "Cannot save notes. Reason:\n$errmsg"
	}\
	else\
	{
		frename $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/notes_${org_id}_${proj_id}_${notes_id}.tmp $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/notes_${org_id}_${proj_id}_${notes_id}
		chmod 0600 $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/notes_${org_id}_${proj_id}_${notes_id}
	}
}

proc notes_export_ascii {org_id proj_id notes_id}\
{
	upvar #0 NOTES_${org_id}_${proj_id}_${notes_id} NOTES
	global FILENAME
	o_wait
	notes_save $org_id $proj_id $notes_id
	set ct [catch {
		set F [open [pwd]/$FILENAME w+]
	} errmsg]
	if {$ct > 0}\
	{
		myError "Cannot save the notes as text. Reason:\n$errmsg"
		return
	}
	puts $F "[keylget NOTES name]\n"
	puts $F "[keylget NOTES notes]"
	close $F
	o_idle
}

proc notes_export_html {org_id proj_id notes_id}\
{
	upvar #0 NOTES_${org_id}_${proj_id}_${notes_id} NOTES
	global FILENAME
	o_wait
	notes_save $org_id $proj_id $notes_id
	if {[file extension $FILENAME] != ".html"}\
	{
		append FILENAME ".html"
	}
	set ct [catch {
		set F [open [pwd]/$FILENAME w+]
	} errmsg]
	if {$ct > 0}\
	{
		myError "Cannot save the notes as text. Reason:\n$errmsg"
		return
	}
	puts $F "<html><head>"
	puts $F "<title>[html_text2html [keylget NOTES name]]</title></head>"
	puts $F "<body bgcolor=\"#FFFFFF\">"
	puts $F "<h2>[html_text2html [keylget NOTES name]]</h2>"
	regsub -all "\n" [keylget NOTES notes] "<br>" notes
	puts $F "[html_text2html $notes]"
	puts $F "<body></html>"
	close $F
	o_idle
}

proc notes_d_export {org_id proj_id notes_id mode}\
{
	global FILENAME fbe 
	catch {destroy .file}
	toplevel .file
	wm title .file "Export notes - $mode"
	wm geometry .file 250x250+100+50
	filebrowser .file.fb 
	pack .file.fb
		if {$mode == "html"}\
		{
			.file.fb addlistbox .*.html -selectmode single
		}\
		else\
		{
			.file.fb addlistbox .* -selectmode single
		}
		.file.fb addlabel -text "Export: "
		.file.fb adddirlabel
		set fbe [.file.fb addentry -width 20]
		.file.fb addbutton -text OK -width 10 \
			-command "
				set FILENAME \[$fbe get\]
				if {\$FILENAME != \"\"}\
				{
					destroy .file
					notes_export_$mode $org_id $proj_id $notes_id
				}
			"
		.file.fb addbutton -text Cancel -width 10 \
			-command "destroy .file"
		if {$mode == "html"}\
		{
			.file.fb readdir [pwd] .*.html
		}\
		else\
		{
			.file.fb readdir [pwd] .*
		}
	focus $fbe
}

proc notes_delete {org_id proj_id notes_id}\
{
	global ORG_PATH LIST_NOTES CUR_NOTES CUR_TYPE
	global NOTES_${org_id}_${proj_id}_${notes_id}
	global confirm
	gen_d_yn_confirm "Do you really want to delete \"[keylget NOTES_${org_id}_${proj_id}_${notes_id} name]\"?"
	tkwait variable confirm
	if {$confirm == "yes"}\
	{
		unlink $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/notes_${org_id}_${proj_id}_${notes_id}
		set LIST_NOTES [lindex [intersect3 $LIST_NOTES ${org_id}_${proj_id}_${notes_id}] 0]
		unset NOTES_${org_id}_${proj_id}_${notes_id}
		o_menu_compose $org_id
		set CUR_TYPE proj
		o_display_proj $org_id $proj_id
	}
}

###########################

proc notes_d_ask_notes {org_id proj_id notes_id}\
{
	upvar #0 NOTES_${org_id}_${proj_id}_${notes_id} NOTES
	global tx
	catch {destroy .ask}
	toplevel .ask
	wm title .ask "Notes"
	wm geometry .ask +100+100
	frame .ask.b
	pack .ask.b -side bottom
		button .ask.b.ok -text Ok -width 10 \
			-command {
				set tx "[.ask.notes get 1.0 end-1c]"
				destroy .ask
			}
		pack .ask.b.ok -side left
		button .ask.b.cancel -text Cancel -width 10
		pack .ask.b.cancel -side left
	scrollbar .ask.vs -command {.ask.notes yview}
	pack .ask.vs -side right -fill y
	text .ask.notes -width 60 -height 12 -yscrollcommand {.ask.vs set} -wrap word
	pack .ask.notes -side top -fill x -anchor w
	if {[keylget NOTES notes {}]}\
	{
		.ask.notes insert 1.0 "[keylget NOTES notes]"
		.ask.b.cancel configure -command "
			set tx [list [keylget NOTES notes]]
			destroy .ask
		"
	}\
	else\
	{
		.ask.b.cancel configure -command "
			set tx \"\"
			destroy .ask
		"
	}
	focus .ask.notes
}

proc notes_popup_notes {org_id proj_id notes_id}\
{
	upvar #0 NOTES_${org_id}_${proj_id}_${notes_id} NOTES
	global tx
	notes_d_ask_notes $org_id $proj_id $notes_id
	tkwait variable tx
	keylset NOTES notes "$tx"
	.r.notes.c itemconfigure notes -text "[keylget NOTES notes]"
	.r.notes.c configure -scrollregion [.r.notes.c bbox all]
}

proc notes_create {org_id proj_id notes_id}\
{
	upvar #0 NOTES_${org_id}_${proj_id}_${notes_id} NOTES
	.r.notes.c delete all
	set tag1 [.r.notes.c create rectangle 0 0 300 30 -fill #EEEEEE -outline {}]
	set tag2 [.r.notes.c create text 50 10 -text "[keylget NOTES name]" -anchor nw]
	set tag3 [.r.notes.c create text 10 50 -text "[keylget NOTES notes]" -width 400 -anchor nw -tags notes]
	bind .r.notes.c <1> "
		notes_popup_notes $org_id $proj_id $notes_id
	"
	.r.notes.c configure -scrollregion [.r.notes.c bbox all]
}
